"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EncoderFull = void 0;
const isUint8Array_1 = require("../../util/isUint8Array");
const v4_1 = require("../Encoder/v4");
const JsonPackExtension_1 = require("../JsonPackExtension");
const JsonPackValue_1 = require("../JsonPackValue");
class EncoderFull extends v4_1.Encoder {
    encodeAny(json) {
        switch (json) {
            case null: return this.u8(0xc0);
            case false: return this.u8(0xc2);
            case true: return this.u8(0xc3);
        }
        if (json instanceof Array)
            return this.encodeArray(json);
        switch (typeof json) {
            case 'number': return this.encodeNumber(json);
            case 'string': return this.encodeString(json);
            case 'object': {
                if (json instanceof JsonPackValue_1.JsonPackValue)
                    return this.buf(json.buf, json.buf.byteLength);
                if (json instanceof JsonPackExtension_1.JsonPackExtension)
                    return this.ext(json);
                if (isUint8Array_1.isUint8Array(json))
                    return this.bin(json);
                return this.encodeObject(json);
            }
        }
    }
    bin(buf) {
        const length = buf.byteLength;
        if (length <= 0xFF)
            this.u16((0xc4 << 8) | length);
        else if (length <= 0xFFFF) {
            this.u8(0xc5);
            this.u16(length);
        }
        else if (length <= 0xFFFFFFFF) {
            this.u8(0xc6);
            this.u32(length);
        }
        this.buf(buf, length);
    }
    ext(ext) {
        const { type, buf } = ext;
        const length = buf.byteLength;
        switch (length) {
            case 1: return this.ext0(buf, (0xd4 << 8) | type, 1);
            case 2: return this.ext0(buf, (0xd5 << 8) | type, 2);
            case 4: return this.ext0(buf, (0xd6 << 8) | type, 4);
            case 8: return this.ext0(buf, (0xd7 << 8) | type, 8);
            case 16: return this.ext0(buf, (0xd8 << 8) | type, 16);
        }
        if (length <= 0xFF) {
            this.u16((0xc7 << 8) | length);
            this.u8(type);
        }
        else if (length <= 0xFFFF) {
            this.u8(0xc8);
            this.u16(length);
            this.u8(type);
        }
        else if (length <= 0xFFFFFFFF) {
            this.u8(0xc9);
            this.u32(length);
            this.u8(type);
        }
        this.buf(buf, length);
    }
    ext0(buf, firstTwo, length) {
        this.u16(firstTwo);
        this.buf(buf, length);
    }
    buf(buf, length) {
        this.ensureCapacity(length);
        this.uint8.set(buf, this.offset);
        this.offset += length;
    }
}
exports.EncoderFull = EncoderFull;
