"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClockEncoder = void 0;
var tslib_1 = require("tslib");
var RelativeTimestamp_1 = require("./RelativeTimestamp");
var ClockEncoder = (function () {
    function ClockEncoder(clock) {
        this.clock = clock;
        this.index = 1;
        this.table = new Map();
        this.table.set(clock.sessionId, this.index++);
    }
    ClockEncoder.prototype.append = function (ts) {
        var sessionId = ts.sessionId, time = ts.time;
        if (sessionId === 0)
            return new RelativeTimestamp_1.RelativeTimestamp(0, ts.time);
        var clock = this.clock.clocks.get(sessionId);
        if (!clock)
            throw new Error("Clock not found (" + sessionId + ", " + time + ").");
        if (!this.table.has(sessionId))
            this.table.set(sessionId, this.index++);
        var sessionIndex = this.table.get(sessionId);
        var timeDiff = clock.time - time;
        var relativeTimestamp = new RelativeTimestamp_1.RelativeTimestamp(sessionIndex, timeDiff);
        return relativeTimestamp;
    };
    ClockEncoder.prototype.toJson = function () {
        var e_1, _a;
        var out = [];
        try {
            for (var _b = tslib_1.__values(this.table.keys()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var sessionId = _c.value;
                var clock = this.clock.clocks.get(sessionId);
                if (!clock)
                    continue;
                out.push(clock.sessionId, clock.time);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return out;
    };
    ClockEncoder.prototype.compact = function () {
        var e_2, _a;
        var isFirst = true;
        var str = '[';
        try {
            for (var _b = tslib_1.__values(this.table.keys()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var sessionId = _c.value;
                var clock = this.clock.clocks.get(sessionId);
                if (!clock)
                    continue;
                str += (isFirst ? '' : ',') + clock.compact();
                isFirst = false;
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return str + ']';
    };
    return ClockEncoder;
}());
exports.ClockEncoder = ClockEncoder;
